# Generated by Django 4.2.23 on 2025-09-03 03:14
from django.core.management import call_command
from django.db import migrations
from django.db.backends.base.schema import BaseDatabaseSchemaEditor
from django.db.migrations.state import StateApps


def create_default_system_info_types(
    apps: StateApps, schema_editor: BaseDatabaseSchemaEditor
) -> None:
    call_command("create_default_system_message_types")


class Migration(migrations.Migration):

    dependencies = [
        ("system_message", "0001_add_system_message_model"),
    ]

    operations = [
        migrations.RunPython(
            create_default_system_info_types, migrations.RunPython.noop
        )
    ]
